@@ -1,9 +1,11 @@
 from dataclasses import dataclass, field
 from functools import partial
+from os import listdir
 from pathlib import Path
 from typing import Any, Container, Sequence
 from typing import Callable, Dict, Optional, Tuple, List, Mapping
 
+import joblib
 import numpy as np
 import torch
 from catalyst.callbacks import BatchMetricCallback
@@ -14,7 +16,8 @@ from torch import nn
 from torch.nn import DataParallel
 from torch.utils.data import Dataset, SequentialSampler
 
-from dnn_cool.utils.base import any_value
+from dnn_cool.memmap.base import RaggedMemoryMap
+from dnn_cool.utils.base import any_value, squeeze_last_axis_if_needed
 
 
 def publish_all(prefix: str,
@@ -215,6 +218,7 @@ class InterpretationCallback(Callback):
 
     def __init__(self, per_sample_criterion,
                  tensorboard_converters: Optional[TensorboardConverters] = None,
+                 infer_logdir=None,
                  loaders_to_skip=()):
         """
         :param flow: The task flow, which holds the per sample loss functions for every task.
@@ -231,7 +235,7 @@ class InterpretationCallback(Callback):
         self.leaf_losses = self.overall_loss.get_leaf_losses_per_sample()
         self.interpretations = {}
         self.loader_counts = {}
-
+        self.infer_logdir = infer_logdir
         self.tensorboard_converters = tensorboard_converters
 
     def _initialize_interpretations(self):
@@ -247,8 +251,8 @@ class InterpretationCallback(Callback):
     def on_loader_start(self, state: State):
         if should_skip_loader(state, self.loaders_to_skip):
             return
-        self.interpretations[state.loader_name] = self._initialize_interpretations()
-        self.loader_counts[state.loader_name] = 0
+        self.interpretations[state.loader_key] = self._initialize_interpretations()
+        self.loader_counts[state.loader_key] = 0
 
         if self.tensorboard_converters is not None:
             self.tensorboard_converters.initialize(state)
@@ -259,31 +263,38 @@ class InterpretationCallback(Callback):
         outputs = state.output['logits']
         targets = state.input['targets']
         overall_res = self.overall_loss(outputs, targets)
-        start = self.loader_counts[state.loader_name]
+        start = self.loader_counts[state.loader_key]
 
         n = 0
         for path, loss in overall_res.items():
             if path.startswith('indices'):
                 continue
-            self.interpretations[state.loader_name][path].append(loss.detach().cpu().numpy())
+            self.interpretations[state.loader_key][path].append(loss.detach().cpu().numpy())
             ind_key = f'indices|{path}'
             indices = overall_res[ind_key] + start
-            self.interpretations[state.loader_name][ind_key].append(indices.detach().cpu().numpy())
+            self.interpretations[state.loader_key][ind_key].append(indices.detach().cpu().numpy())
             n = len(indices)
 
-        self.loader_counts[state.loader_name] += n
+        self.loader_counts[state.loader_key] += n
 
     def on_loader_end(self, state: State):
         if should_skip_loader(state, self.loaders_to_skip):
             return
-        self.interpretations[state.loader_name] = self.prepare_interpretations(state)
+        self.interpretations[state.loader_key] = self.prepare_interpretations(state)
 
         if self.tensorboard_converters is not None:
-            self.tensorboard_converters.publish(state, self.interpretations[state.loader_name])
+            self.tensorboard_converters.publish(state, self.interpretations[state.loader_key])
+        nested_dict = self.interpretations[state.loader_key]
+        self.interpretations[state.loader_key] = to_dict_of_lists(self.interpretations[state.loader_key],
+                                                                  nested_dict,
+                                                                  self.infer_logdir,
+                                                                  'interpretations',
+                                                                  state.loader_key,
+                                                                  interpretation_mode=True)
 
     def prepare_interpretations(self, state):
         res = {}
-        for key, value in self.interpretations[state.loader_name].items():
+        for key, value in self.interpretations[state.loader_key].items():
             arrs = []
             for arr in value:
                 try:
@@ -591,9 +602,84 @@ class SingleLossInterpretationCallback(IMetricCallback):
         else:
             indices = runner.input[self._idx_key].detach().cpu().numpy()
 
-        self.interpretations[runner.loader_name]["loss"].append(loss_items.detach().cpu().numpy())
-        self.interpretations[runner.loader_name]["indices"].append(indices)
+        self.interpretations[runner.loader_key]["loss"].append(loss_items.detach().cpu().numpy())
+        self.interpretations[runner.loader_key]["indices"].append(indices)
 
     @property
     def metric_fn(self):
         return self.metric
+
+
+def to_dict_of_lists(nested_dict, precondition_dict, parent_dir, name, loader_key, interpretation_mode=False):
+    res = {}
+    for key, value in nested_dict.items():
+        if key.startswith('precondition'):
+            continue
+        if parent_dir is None:
+            continue
+        out_dir = parent_dir / loader_key
+        out_dir.mkdir(exist_ok=True)
+        out_dir = (out_dir / name)
+        out_dir.mkdir(exist_ok=True)
+        if interpretation_mode:
+            joblib.dump(value, out_dir / f'{key}.pkl')
+            res[key] = value
+            continue
+        valid_indices, valid_values = concat_nested(key, value, precondition_dict)
+        res[key] = valid_values
+        joblib.dump(valid_values, out_dir / f'{key}.pkl')
+        indices_dir = (parent_dir / loader_key / 'indices')
+        indices_dir.mkdir(exist_ok=True)
+        joblib.dump(valid_indices, indices_dir / f'{key}.pkl')
+    return res
+
+
+def concat_nested(key, value, precondition_dict):
+    valid_values = []
+    valid_indices = []
+    offset = 0
+    for i in range(len(value)):
+        arr = value[i]
+        preconditions = precondition_dict.get(f'precondition|{key}')
+        precondition = preconditions[i] if preconditions is not None else np.ones_like(arr, dtype=bool)
+        precondition = squeeze_last_axis_if_needed(precondition)
+        valid_values.append(arr[precondition])
+        valid_axes = np.where(precondition)
+        for axis, nonzero in enumerate(valid_axes):
+            if not (axis < len(valid_indices)):
+                valid_indices.append([])
+            if axis == 0:
+                nonzero += offset
+                offset += len(arr)
+            valid_indices[axis].append(nonzero)
+    valid_values = np.concatenate(valid_values, axis=0)
+    valid_indices = squeeze_last_axis_if_needed(np.stack([np.concatenate(v) for v in valid_indices]).T)
+    return valid_indices, valid_values
+
+
+def try_to_concat_list(list_of_samples, v):
+    try:
+        for sample in v:
+            list_of_samples.append(sample)
+    except:
+        list_of_samples.append(v)
+
+
+def load_inference_results_from_directory(logdir):
+    out_dir = logdir / 'infer'
+    out_dir.mkdir(exist_ok=True)
+    res = {}
+    for key in ['logits', 'targets', 'interpretations']:
+        res[key] = {}
+        for loader_name in ['infer', 'valid', 'test']:
+            res[key][loader_name] = {}
+            for filename in listdir(out_dir / loader_name / key):
+                if not filename.endswith('.pkl'):
+                    continue
+                full_path = out_dir / loader_name / key / filename
+                task_name = full_path.stem
+                res[key][loader_name][task_name] = joblib.load(full_path)
+                if key == 'logits' and not task_name.startswith('indices|'):
+                    indices_path = out_dir / loader_name / 'indices' / filename
+                    res[key][loader_name][f'indices|{task_name}'] = joblib.load(indices_path)
+    return res
